unsigned long
pow(unsigned long b,
    unsigned short e) {
  if (e == 0)
    return 1;
  auto rec = pow(b, e/2);
  return ((e%2)?b:1)*rec*rec;
}

void
fermat(unsigned long x, unsigned long y, unsigned long z,
       unsigned short n) {
  if ((n > 2) && (x > 0) && (y > 0))
    axiom_assert(pow(x, n) + pow(y, n) != pow(z, n));
}

bool
simple_test() {
  auto generator =
    list_data_generator<unsigned long, unsigned short>(
      {1ul, 2ul, 3ul, 4ul, 5ul, 6ul},
      {3ul, 4ul, 5ul, 6ul});

  if (!test(generator, fermat, "fermat"))
    return false;

  return true;
}

bool
random_test(std::mt19937& engine) {
  auto generator =
    term_generator_builder<unsigned long, unsigned short>(10u)
    (engine,
     std::function<unsigned long()>
     ([&engine] () {
       return std::uniform_int_distribution<unsigned long>()(engine);
     }),
     std::function<unsigned short()>
     ([&engine] () {
       return std::uniform_int_distribution<unsigned short>()(engine);
     }));

  if (!test(generator, fermat, "fermat random"))
    return false;

  return true;
}
