// This file is a part of Catsfoot.
// Copyright (C) 2011  Uni Research
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser Public License for more details.
//
// You should have received a copy of the GNU Lesser Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef CONCEPTS_HH
# define CONCEPTS_HH

# include "../type_traits/is_callable.hh"
# include "concept_tools.hh"
# include <type_traits>

namespace catsfoot {

  /// \brief Check whether \a T is comparable to \a U by ==
  /// \ingroup Concepts
  template <typename T, typename U = T>
  struct equality: public auto_concept {
  typedef concept_list<
    is_callable<op_eq(T, U)>,
    std::is_convertible<typename is_callable<op_eq(T, U)>::result_type,
			bool>
    > requirements;
  };

  /// \brief Check whether \a U is printable on a stream \a T
  /// \ingroup Concepts
  template <typename T, typename U>
  struct printable: public auto_concept {
  typedef concept_list<
    is_callable<op_lsh(T, U)>
    > requirements;
  };

  namespace details {
    template <typename T, typename U>
    struct is_same: public std::false_type {
    };

    template <typename T>
    struct is_same<T, T>: public std::true_type {
    };
  }

  /// \brief Check whether \a T and \a U are the same type
  /// \ingroup Concepts
  template <typename T, typename U>
  struct is_same: public details::is_same<T,U> {};

}

#endif
