// This file is a part of Catsfoot.
// Copyright (C) 2011  Uni Research
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser Public License for more details.
//
// You should have received a copy of the GNU Lesser Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

# include <deque>
# include <sstream>
# define __CATSFOOT_ENABLE_TESTING_RT 1
# include "parse_ids.hh"

namespace catsfoot {

  namespace details {

    /// \brief Used for stream operation. Encapsulates a string.
    struct identifier {
      std::string s;
    };

    template <typename Stream>
    static Stream& operator>>(Stream& s, identifier& i) {
      i.s = "";
      char c;
      std::deque<char> stack;
      do {
	s >> c;
      } while (::isspace(c));

      do {
	if (stack.empty() && (c == ','))
	  break ;
	switch (c) {
	case '<':
	  stack.push_back('<'); break ;
	case '>':
	  if (!stack.empty())
	    if (stack.back() == '<')
	      stack.pop_back();
	  break ;
	case '(':
	  stack.push_back('('); break ;
	case ')':
	  while (!stack.empty()) {
	    if (stack.back() == '(') {
	      stack.pop_back();
	      break ;
	    }
	    else
	      stack.pop_back();
	  }
	}
	if (!isspace(c))
	  i.s.append(1, c);
      } while (s >> c);
      return s;
    }

    /// \brief Splits a comma seperated list of C++ identifiers into a
    /// vector
    std::vector<std::string>
    split_identifiers(const std::string& l) {
      std::istringstream ss(l);
      std::vector<std::string> ret;
      while (!ss.eof()) {
	identifier i;
	ss >> i;
	ret.push_back(i.s);
      }
      return ret;
    }

  }
}
