// This file is a part of Catsfoot.
// Copyright (C) 2011  Uni Research
// Copyright (C) 2011  Valentin David
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser Public License for more details.
//
// You should have received a copy of the GNU Lesser Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef PARSE_IDS_HH
# define PARSE_IDS_HH

# include <vector>
# include <tuple>
# include <string>
# include "../type_traits/always_false.hh"

namespace catsfoot {

  namespace details {

    /// \brief Zip a vector of T with a tuple of U...
    template <typename T, typename... U, typename... V,
	      typename = typename
	      std::enable_if<sizeof...(V) == sizeof...(U)>::type>
    std::tuple<std::pair<T, U>...>
    zip_vec_tuple(const std::vector<T>& vec,
		  const std::tuple<U...>&,
		  V&&... v) {
      assert(vec.size() == sizeof...(V));
      return std::tuple<std::pair<T, U>...>{std::forward<V>(v)...};
    }

    /// \brief Zip a vector of T with a tuple of U...
    template <typename T, typename... U, typename... V,
	      typename = void,
	      typename
# ifndef DOC_GEN
	      = typename
	      std::enable_if<sizeof...(V) < sizeof...(U)>::type
# endif
	     >
    std::tuple<std::pair<T, U>...>
    zip_vec_tuple(const std::vector<T>& v,
		  const std::tuple<U...>& t,
		  V&&... values) {
      return zip_vec_tuple
	(v, t, std::forward<V>(values)...,
	 std::pair
	 <T, typename std::tuple_element<sizeof...(V), std::tuple<U...> >
	  ::type>
	 {v[sizeof...(V)], std::get<sizeof...(V)>(t)});
    }

# ifdef  __CATSFOOT_ENABLE_TESTING_RT
#  include "../extern_decl.hh"

    /// \brief Splits a comma separated list of C++ identifiers into a
    /// vector
    EXTERN
    std::vector<std::string>
    split_identifiers(const std::string&);
# else
    template <typename T = std::string>
    std::vector<std::string>
    split_identifiers(const T&) {
      static_assert(details::always_false<T>::value,
		    "You probably called a functionality that is available "
		    "only if you link with the runtime library. In this case "
		    "include catsfoot_testing.hh instead of catsfoot.hh");
      return std::vector<std::string>{};
    }
# endif
  }
}

#endif
