// This file is a part of Catsfoot.
// Copyright (C) 2011  Uni Research
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser Public License for more details.
//
// You should have received a copy of the GNU Lesser Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef __CHOOSE_HH
# define __CHOOSE_HH

namespace catsfoot {
  namespace details {
    template <typename...>
    struct generator_choose {};

    template <typename T, typename... Other>
    struct generator_choose<T, Other...>
      : public generator_choose<Other...> {
    private:
      typedef generator_choose<Other...> super;
      T generator;

    public:
      generator_choose(T t, Other... other)
	: generator_choose<Other...>(std::forward<Other>(other)...),
	  generator(std::forward<T>(t)) {
      }

      template <typename Return,
		ENABLE_IF(generator_for<T, Return>),
		typename Ret = typename
	      is_callable<member_get(T&, selector<Return>)>::result_type>
      Ret get(selector<Return> s) {
	return generator.get(s);
      }

      template <typename Return,
		ENABLE_IF_NOT(generator_for<T, Return>),
		ENABLE_IF(generator_for<super, Return>),
		typename Ret = typename
		is_callable<member_get(super&,
				       selector<Return>)>::result_type>
      Ret get(selector<Return> s...) {
	return this->super::get(s);
      }
    };
  }

  /// \brief Combines generators.
  /// \ingroup DataGen
  ///
  /// Picks the left-most generator capable or generating the requested
  /// data set.
  template <typename... T>
  details::generator_choose<typename std::decay<T>::type...>
  choose(T&&... t) {
    return details::generator_choose<typename std::decay<T>::type...>
      {std::forward<T>(t)...};
  }

}

#endif
