// This file is a part of Catsfoot.
// Copyright (C) 2011  Uni Research
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser Public License for more details.
//
// You should have received a copy of the GNU Lesser Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef __PICK_HH
# define __PICK_HH

# include "dataset.hh"

namespace catsfoot {
  template <typename T, typename Generator>
  struct pick_functor {
  private:
    typedef typename std::decay<decltype(std::declval<Generator&>().get(selector<T>()))>::type container;
    container s;
    typename container::iterator i;
  public:
    pick_functor(Generator& g): s(g.get(selector<T>())), i(s.begin()) {
    }
    pick_functor(const pick_functor& other): s(other.s), i(s.begin()) {
    }
    pick_functor(pick_functor&& other): s(std::move(other.s)),
					i(std::move(other.i)) {
    }

    pick_functor& operator=(pick_functor&& other) {
      std::swap(s, other.s);
      std::swap(i, other.i);
      return *this;
    }

    pick_functor& operator=(const pick_functor& other) {
      return *this = pick_functor(other);
    }

    T& operator()() {
      if (i == s.end()) {
	i = s.begin();
	assert(i != s.end());
      }
      return *i++;
    }
  };

  template <typename T, typename Generator>
  std::function<T()> pick(Generator& g) {
    return pick_functor<T, Generator>(g);
  }
}

#endif
