// This file is a part of Catsfoot.
// Copyright (C) 2011  Uni Research
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser Public License for more details.
//
// You should have received a copy of the GNU Lesser Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef IS_CONSTRUCTIBLE_HH
# define IS_CONSTRUCTIBLE_HH

# include "always_false.hh"
# include <type_traits>

namespace catsfoot {

  /// \brief Undefined
  template <typename T>
  struct is_constructible: public std::false_type {
    static_assert(details::always_false<T>::value,
		  "Bad format of parameter");
  };

  namespace details {
    /// \brief Work around for some GCC 4.6 alpha
    template <bool B, typename T>
    struct is_constructible_work_around;

    /// \brief If T is not a class, it cannot be built with several parameter
    template <typename T, typename... U>
    struct is_constructible_work_around<false, T(U...)>:
      public std::false_type {
    };

    /// \brief If not a class, is it always default constructible (maybe)
    template <typename T>
    struct is_constructible_work_around<false, T()>:
      public std::true_type {
    };

    /// \brief If is explicitly convertible and not a class, then it is
    /// like constructible.
    template <typename T, typename U>
    struct is_constructible_work_around<false, T(U)>:
      public std::is_explicitly_convertible<U, T> {
    };

    /// \brief If T is a class, then use the standard library
    template <typename T, typename... U>
    struct is_constructible_work_around<true, T(U...)>
      : public std::is_constructible<T, U...> {
    };
  }

#ifndef SHOULDBE
  template <typename T, typename... U>
  struct is_constructible<T(U...)>:
    public details::
    is_constructible_work_around<std::is_class<T>::value, T(U...)> {
    };
#else
  /// \brief Tells whether T is constructible with {U...}
  /// \ingroup Predicates
  template <typename T, typename... U>
  struct is_constructible<T(U...)>:
    public std::is_constructible<T, U...> {
  };

  /// Void is a special type that may induce extra error messages
  template <typename... U>
  struct is_constructible<void(U...)>:
    public std::false_type {};
#endif

}

#endif
