// This file is a part of Catsfoot.
// Copyright (C) 2011  Uni Research
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser Public License for more details.
//
// You should have received a copy of the GNU Lesser Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <dataset/random_term_generator.hh>
#include <iostream>
#include <fstream>
#include <wrappers/function_wrappers.hh>

#define MAKE_INT_TERM(name) \
struct name { \
private: \
  int i; \
public: \
  name(): i(0) {} \
  name(const name& other): i(other.i) {} \
  explicit name(int i): i(i) {} \
 \
  static name one() { \
    name ret; \
    ++ret.i; \
    return ret; \
  } \
 \
  name& operator=(const name& other) { \
    i = other.i; \
    return *this; \
  } \
 \
  bool operator==(const name& other) const { \
    return i == other.i; \
  } \
 \
  name operator+(const name& other) const { \
    name ret(*this); \
    ret.i+=other.i; \
    return ret; \
  } \
 \
  operator int() const { \
    return i; \
  } \
  \
  name non_const() { \
    return *this; \
  } \
}; \
 \
template <typename Stream> \
Stream& operator<<(Stream& s, const name& t) { \
  return (s << int(t)); \
}

MAKE_INT_TERM(some_point);
MAKE_INT_TERM(some_branch);

some_point point_something(const some_point& a, const some_branch&) {
  some_point ret(int(a)+1);
  return ret;
}

some_branch branch_something(const some_point&, const some_branch& a) {
  some_branch ret(int(a)+1);
  return ret;
}

std::mt19937 engine;

auto point_branch_generator =
  catsfoot::term_generator_builder<some_point, some_branch>()
(engine,
 std::function<some_point()>([] () {
     return some_point(engine()); }),
 catsfoot::constructor<some_point()>(),
 point_something,
 &some_point::non_const,
 std::function<some_branch()>([] () {
     return some_branch(engine()); }),
 catsfoot::constructor<some_branch()>(),
 branch_something);

int main() {
  uint_fast32_t seed;
  {
    std::ifstream urandom("/dev/urandom");
    urandom.read((char*)&seed, sizeof(uint_fast32_t));
  }
  engine.seed(seed);

  auto points = point_branch_generator.get(catsfoot::selector<some_point>{});
  auto i = points.begin();
  for (;
       (i != points.end());
       ++i) {
    std::cout << *i
	      << std::endl;
  }

  return 0;
}
